function IOSig = impulse_noise(IOSig, Ps)
% IMPULSE_NOISE generates randomly spaced amplitude bursts 
%
% OSig = IMPULSE_NOISE(ISig, Ps)
% INPUT
%    ISig - Input signal
%      Ps - Probability 0<=Ps<=1 for each sample to be superimposed by an impulse (default: 0.01)
% OUTPUT
%    OSig - Output signal (input signal with superimposed bursts)  

%   Rev 1.0     Oct. 06, 2002   Dmitiry Shutin <dshutin@inw.tugraz.at>
%   Rev 1.1     Oct. 16, 2002   Multidimensional input support
%   Rev 2.0     Oct. 19, 2009   Daniel Arnitz <daniel.arnitz@tugraz.at>
%                               complete redesign; made number Ns -> probability Ps
%   Rev 2.1     Oct. 30, 2009   check for numeric IOSig, typecast to double and back to original type
% *******************************************************************************************************

% input parameter checks
if nargin < 1
   IOSig = [];
end
if nargin < 2
   Ps = 0.01;
end
if ~isnumeric(IOSig)
   error('ISig has to be a numeric array.');
end

% make sure this works for all vector/matrix orientations and (numeric) classes
orig_size = size(IOSig);
orig_type = class(IOSig);
IOSig = double(IOSig(:));

% add noise
%     linear boolean index of samples that will be altered (0: do not alter, 1: alter)
ind = rand(numel(IOSig), 1) < Ps;
%     randomly modify IOSig's amplitude at all ind==1
Ampl = max(IOSig) - min(IOSig);
if Ampl==0; Ampl = 1; end
IOSig(ind) = randn(sum(ind), 1) .* Ampl + IOSig(ind);

% reshape to original size and typecast to original class
IOSig = feval(orig_type, reshape(IOSig, orig_size));
